==> blcr-0.2.0.tar.gz: blcr-0.2.0/libcr/cr_private.h <==
/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_private.h,v 1.63 2003/11/14 19:55:24 phargrov Exp $
 */

#ifndef _CR_PRIVATE_H
#define _CR_PRIVATE_H	1

#include "config.h"
#include <features.h>

#include <sys/types.h>
#include <signal.h>
#include <pthread.h>
#include <libcr.h>
#include <cr.h>
#include "cr_trace.h"
#include "cr_atomic.h"
#include "cr_yield.h"
#include "cr_rb_lock.h"

typedef void (*cri_sighandler_t)(int sig, siginfo_t *info, void *context);

typedef struct cri_info_s {
    /* Current state */
    cri_atomic_t	cr_state;

    /* Count of critical sections */
    cri_atomic_t	cr_cs_count;

    /* Vector of registered checkpoint callbacks and their private data */
    unsigned int	cr_cb_count;
    struct {
	cr_callback_t		func;	// The function to invoke
	void*			arg;	// The argument to the function
	int			flags;	// The flags
    }			cr_cb[CR_MAX_CALLBACKS];

    /* What identifier to return next from cr_init() */
    cr_client_id_t	next_id;

    /* What signal handler to run for this thread? */
    cri_sighandler_t	handler;

    /* XXX: ioctl number to reregister handler with the kernel.  This is
     * needed in cr_syscall() as a work-around until the kernel does it.
     */
    unsigned long	register_cmd;

    /* Data needed to invoke all the callbacks in order */
    struct {
	int			depth;
	int			index;
	int			rc;
    }			run;

    /* Thread-local info about the checkpoint request */
    struct cr_checkpoint_info	cr_checkpoint_info;

    /* Data needed to start a new thread */
    void		*(*thread_fcn)(void *);
    void		*thread_arg;
} cri_info_t;

// Indicates if the pthreads library has been linked in
extern int cri_pthread_present;

// Pointer to the cr_info for the main thread
extern cri_info_t *cri_info_main_p;

// How many threads are actively trying to checkpoint (PENDING + ACTIVE)
extern cri_atomic_t cri_live_count;

// Red-black lock for critical sections
extern cri_rb_lock_t cri_cs_lock;

// Thread-specific data key for cr_info
extern pthread_key_t cri_info_key;

typedef int (*cri_pthread_create_t)(pthread_t *thread,
				    const pthread_attr_t *attr,
				    void * (*start_routine)(void *),
				    void *arg);

// Holds address of pthread_create(), on which we interpose.
extern cri_pthread_create_t cri_real_pthread_create;

// Location of the (possibly thread-specific) cr_info
#define cri_info_location()	\
	(cri_pthread_present ? (cri_info_t *)pthread_getspecific(cri_info_key) \
			     : cri_info_main_p)

// Start the checkpoint
extern void cri_start_checkpoint(cri_info_t *info, int is_thread);

// Perform pthreads-related initialization if needed.
extern void cri_pthread_init(void);

// Register a callback
extern cr_callback_id_t cri_do_register(cri_info_t*, cr_callback_t, void*, int);
extern cr_callback_id_t cri_register_thread(cri_info_t*, cr_callback_t, void*, int);

// Replace a callback
extern int cri_do_replace(cri_info_t*, cr_callback_id_t, cr_callback_t, void*, int);
extern int cri_replace_thread(cri_info_t*, cr_callback_id_t, cr_callback_t, void*, int);

// Create cri_info_t
extern cri_info_t* cri_info_init(void);

// Initialize checkpoint info
extern void cri_get_siginfo(cri_info_t *info, siginfo_t *siginfo);

// cr_syscall.c
#define CRI_SYSCALL_NOARG ((void *)(-1L))
extern int cri_get_fd(void);
extern int cri_syscall(unsigned long int op, void * arg);
extern void cri_fd_checkpoint(void); // XXX: temporary
extern void cri_fd_restart(void); // XXX: temporary
extern void cri_fd_continue(void); // XXX: temporary

// cr_sig_sync.c
extern int cri_barrier_enter(cri_atomic_t *x);

// Calls needed to bypass normal linker resolution:
extern pid_t __fork(void);
extern pid_t __vfork(void);
extern int __libc_open(const char *pathname, int flags, ...);
extern int __libc_close(int fd);
extern int __libc_allocate_rtsig(int);
extern int __libc_nanosleep(const struct timespec *, struct timespec *);
#if HAVE___LIBC_SIGACTION
extern int __libc_sigaction(int, const struct sigaction *, struct sigaction *);
#elif HAVE___SIGACTION
extern int __sigaction(int, const struct sigaction *, struct sigaction *);
#endif

// Magic voodoo to produce weak aliases with gcc > 2.7
#ifndef weak_alias
# define weak_alias(name, aname) \
  extern __typeof (name) aname __attribute__ ((weak, alias (#name)));
#endif

// TEMPORARY HACKS:

#if CRI_PTHREAD_HACK
extern void cri_pthread_hack_checkpoint(void);
extern void cri_pthread_hack_continue(void);
extern void cri_pthread_hack_restart(void);
#endif /* CRI_PTHREAD_HACK */

#endif

==> blcr-0.5.0.tar.gz: blcr-0.5.0/libcr/cr_private.h <==
/* 
 * Berkeley Lab Checkpoint/Restart (BLCR) for Linux is Copyright (c)
 * 2003, The Regents of the University of California, through Lawrence
 * Berkeley National Laboratory (subject to receipt of any required
 * approvals from the U.S. Dept. of Energy).  All rights reserved.
 *
 * Portions may be copyrighted by others, as may be noted in specific
 * copyright notices within specific files.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: cr_private.h,v 1.85 2007/02/13 22:45:14 phargrov Exp $
 */

#ifndef _CR_PRIVATE_H
#define _CR_PRIVATE_H	1

#include "blcr_config.h"
#include <features.h>

#include <sys/types.h>
#include <signal.h>		// for siginfo_t
#include <pthread.h>
#include <limits.h>		// for PATH_MAX
#include <libcr.h>
#include "cr_syscall.h"
#include "cr_trace.h"
#include "cr_atomic.h"
#include "cr_yield.h"
#include "cr_rb_lock.h"
#include "cr_arch.h"

typedef void (*cri_sighandler_t)(int sig, siginfo_t *info, void *context);

typedef struct cri_info_s {
    /* Current state */
    cri_atomic_t	cr_state;

    /* Count of critical sections */
    cri_atomic_t	cr_cs_count;

    /* Vector of registered checkpoint callbacks and their private data */
    unsigned int	cr_cb_count;
    struct {
	cr_callback_t		func;	// The function to invoke
	void*			arg;	// The argument to the function
	int			flags;	// The flags
    }			cr_cb[CR_MAX_CALLBACKS];

    /* What identifier to return next from cr_init() */
    cr_client_id_t	next_id;

    /* What signal handler to run for this thread? */
    cri_sighandler_t	handler;

    /* Data needed to invoke all the callbacks in order */
    struct {
	int			index;	/* Counts down only */
	int			id;	/* Tracks id of running callback (down and up again) */
	int			rc;	/* Saves the return code */
    }			run;

    /* Thread-local info about the checkpoint request */
    struct cr_checkpoint_info	cr_checkpoint_info;

    /* Thread-local info about the restart request */
    struct cr_restart_info	cr_restart_info;

    /* Space for the cr_checkpoint_info.dest and cr_restart_info.src */
    char		path[PATH_MAX];
} cri_info_t;

// How many threads are actively trying to checkpoint (PENDING + ACTIVE)
extern cri_atomic_t cri_live_count;	// GLOBAL

// Red-black lock for critical sections
extern cri_rb_lock_t cri_cs_lock;	// GLOBAL

// Thread-specific data key for cr_info
extern pthread_key_t cri_info_key;	// GLOBAL

// Location of the (possibly thread-specific) cr_info
#define cri_info_location()	((cri_info_t *)pthread_getspecific(cri_info_key))

// Start the checkpoint
extern void cri_start_checkpoint(cri_info_t *info, int is_thread);

// Perform pthreads-related initialization if needed.
extern void cri_pthread_init(void);
#if PIC && HAVE___REGISTER_ATFORK
  extern int cri_pthread_atfork(void (*prepare)(void), void (*parent)(void), void (*child)(void));
#else
  #define cri_pthread_atfork pthread_atfork
#endif

// Register a callback
extern cr_callback_id_t cri_do_register(cri_info_t*, cr_callback_t, void*, int);
extern cr_callback_id_t cri_register_thread(cri_info_t*, cr_callback_t, void*, int);

// Replace a callback
extern int cri_do_replace(cri_info_t*, cr_callback_id_t, cr_callback_t, void*, int);
extern int cri_replace_thread(cri_info_t*, cr_callback_id_t, cr_callback_t, void*, int);

// Create or destroy cri_info_t
extern cri_info_t* cri_info_init(void);
extern void cri_info_free(void *);

// Initialize checkpoint info
extern void cri_checkpoint_info_init(cri_info_t *info, siginfo_t *siginfo);

// cr_sig_sync.c
extern int cri_barrier_enter(cri_atomic_t *x);

// libc/libpthread internal routines:
extern int __libc_allocate_rtsig(int);
extern pid_t __fork(void);

// Magic voodoo to produce weak aliases with gcc > 2.7
#ifndef weak_alias
# define weak_alias(name, aname) \
  extern __typeof (name) aname __attribute__ ((weak, alias (#name)));
#endif

#endif
